@@ -29,6 +29,12 @@ import subprocess
 import sys
 from collections import defaultdict, namedtuple
 
+try:
+    zip_longest = itertools.zip_longest
+except AttributeError:
+    # Python 2 fallback
+    zip_longest = itertools.izip_longest
+
 from reportlab.pdfbase.pdfmetrics import registerFont, stringWidth
 from reportlab.pdfbase.ttfonts import TTFError, TTFont
 from reportlab.pdfgen.canvas import FILL_EVEN_ODD, FILL_NON_ZERO
@@ -49,10 +55,10 @@ from .utils import (
     normalise_svg_path,
 )
 
-__version__ = '0.9.2'
+__version__ = '0.9.3'
 __license__ = 'LGPL 3'
 __author__ = 'Dinu Gherman'
-__date__ = '2019-07-12'
+__date__ = '2019-11-02'
 
 XML_NS = 'http://www.w3.org/XML/1998/namespace'
 
@@ -208,19 +214,26 @@ class AttributeConverter(object):
 
         # This needs also to lookup values like "url(#SomeName)"...
 
-        if self.css_rules is not None and not svgNode.attrib.get('__rules_applied', False):
-            if isinstance(svgNode, NodeTracker):
-                svgNode.apply_rules(self.css_rules)
-            else:
-                ElementWrapper(svgNode).apply_rules(self.css_rules)
+        if not svgNode.attrib.get('__rules_applied', False):
+            # Apply global styles...
+            if self.css_rules is not None:
+                if isinstance(svgNode, NodeTracker):
+                    svgNode.apply_rules(self.css_rules)
+                else:
+                    ElementWrapper(svgNode).apply_rules(self.css_rules)
+            # ...and locally defined
+            if svgNode.attrib.get("style"):
+                attrs = self.parseMultiAttributes(svgNode.attrib.get("style"))
+                for key, val in attrs.items():
+                    # lxml nodes cannot accept attributes starting with '-'
+                    if not key.startswith('-'):
+                        svgNode.attrib[key] = val
+                svgNode.attrib['__rules_applied'] = '1'
+
         attr_value = svgNode.attrib.get(name, '').strip()
 
         if attr_value and attr_value != "inherit":
             return attr_value
-        elif svgNode.attrib.get("style"):
-            dict = self.parseMultiAttributes(svgNode.attrib.get("style"))
-            if name in dict:
-                return dict[name]
         if svgNode.getparent() is not None:
             return self.findAttr(svgNode.getparent(), name)
         return ''
@@ -308,12 +321,15 @@ class Svg2RlgAttributeConverter(AttributeConverter):
     def convertLength(self, svgAttr, em_base=12, attr_name=None, default=0.0):
         "Convert length to points."
 
-        text = svgAttr
+        text = svgAttr.replace(',', ' ').strip()
         if not text:
             return default
-        if ' ' in text.replace(',', ' ').strip():
-            logger.debug("Only getting first value of %s" % text)
-            text = text.replace(',', ' ').split()[0]
+        if ' ' in text:
+            # Multiple length values, returning a list
+            return [
+                self.convertLength(val, em_base=em_base, attr_name=attr_name, default=default)
+                for val in self.split_attr_list(text)
+            ]
 
         if text.endswith('%'):
             if self.main_box is None:
@@ -372,30 +388,27 @@ class Svg2RlgAttributeConverter(AttributeConverter):
 
         if text == "currentColor":
             return "currentColor"
-        elif len(text) == 7 and text[0] == '#':
-            return self.color_converter(colors.HexColor(text))
+        if len(text) in (7, 9) and text[0] == '#':
+            color = colors.HexColor(text, hasAlpha=len(text) == 9)
         elif len(text) == 4 and text[0] == '#':
-            return self.color_converter(colors.HexColor('#' + 2*text[1] + 2*text[2] + 2*text[3]))
-        elif text.startswith('rgb') and '%' not in text:
-            t = text[3:].strip('()')
-            tup = [h[2:] for h in [hex(int(num)) for num in t.split(',')]]
-            tup = [(2 - len(h)) * '0' + h for h in tup]
-            col = "#%s%s%s" % tuple(tup)
-            return self.color_converter(colors.HexColor(col))
-        elif text.startswith('rgb') and '%' in text:
-            t = text[3:].replace('%', '').strip('()')
-            tup = (float(val)/100.0 for val in t.split(','))
-            return self.color_converter(colors.Color(*tup))
+            color = colors.HexColor('#' + 2*text[1] + 2*text[2] + 2*text[3])
+        elif len(text) == 5 and text[0] == '#':
+            color = colors.HexColor(
+                '#' + 2*text[1] + 2*text[2] + 2*text[3] + 2*text[4], hasAlpha=True
+            )
         else:
-            # Test if text is a predefined color constant
-            try:
-                color = getattr(colors, text)
-            except AttributeError:
-                logger.warning("Can't handle color: %s" % text)
-            else:
-                return self.color_converter(color)
-
-        return None
+            # Should handle pcmyk|cmyk|rgb|hsl values (including 'a' for alpha)
+            color = colors.cssParse(text)
+            if color is None:
+                # Test if text is a predefined color constant
+                try:
+                    color = getattr(colors, text)
+                except AttributeError:
+                    pass
+        if color is None:
+            logger.warning("Can't handle color: %s" % text)
+        else:
+            return self.color_converter(color)
 
     def convertLineJoin(self, svgAttr):
         return {"miter":0, "round":1, "bevel":2}[svgAttr]
@@ -1055,18 +1068,14 @@ class Svg2RlgShapeConverter(SvgShapeConverter):
             has_x, has_y = False, False
             dx, dy = 0, 0
             baseLineShift = 0
-            if node_name(c) == 'text':
-                text = self.clean_text(c.text, preserve_space)
-                if not text:
-                    continue
-            elif node_name(c) == 'tspan':
+            if node_name(c) in ('text', 'tspan'):
                 text = self.clean_text(c.text, preserve_space)
                 if not text:
                     continue
                 x1, y1, dx, dy = self.convert_length_attrs(c, 'x', 'y', 'dx', 'dy', em_base=fs)
                 has_x, has_y = (c.attrib.get('x', '') != '', c.attrib.get('y', '') != '')
-                dx0 = dx0 + dx
-                dy0 = dy0 + dy
+                dx0 = dx0 + (dx[0] if isinstance(dx, list) else dx)
+                dy0 = dy0 + (dy[0] if isinstance(dy, list) else dy)
                 baseLineShift = c.attrib.get("baseline-shift", '0')
                 if baseLineShift in ("sub", "super", "baseline"):
                     baseLineShift = {"sub":-fs/2, "super":fs/2, "baseline":0}[baseLineShift]
@@ -1076,14 +1085,48 @@ class Svg2RlgShapeConverter(SvgShapeConverter):
                 continue
 
             frag_lengths.append(stringWidth(text, ff, fs))
-            new_x = (x1 + dx) if has_x else (x + dx0 + sum(frag_lengths[:-1]))
-            new_y = (y1 + dy) if has_y else (y + dy0)
-            shape = String(new_x, -(new_y - baseLineShift), text)
-            self.applyStyleOnShape(shape, node)
-            if node_name(c) == 'tspan':
-                self.applyStyleOnShape(shape, c)
 
-            gr.add(shape)
+            # When x, y, dx, or dy is a list, we calculate position for each char of text.
+            if any(isinstance(val, list) for val in (x1, y1, dx, dy)):
+                #if isinstance(dx, list):
+                #    import pdb; pdb.set_trace()
+                if has_x:
+                    xlist = x1 if isinstance(x1, list) else [x1]
+                else:
+                    xlist = [x + dx0 + sum(frag_lengths[:-1])]
+                if has_y:
+                    ylist = y1 if isinstance(y1, list) else [y1]
+                else:
+                    ylist = [y + dy0]
+                dxlist = dx if isinstance(dx, list) else [dx]
+                dylist = dy if isinstance(dy, list) else [dy]
+                last_x, last_y, last_char = xlist[0], ylist[0], ''
+                for char_x, char_y, char_dx, char_dy, char in zip_longest(
+                        xlist, ylist, dxlist, dylist, text):
+                    if char is None:
+                        break
+                    if char_dx is None:
+                        char_dx = 0
+                    if char_dy is None:
+                        char_dy = 0
+                    new_x = char_dx + (last_x + stringWidth(last_char, ff, fs) if char_x is None else char_x)
+                    new_y = char_dy + (last_y if char_y is None else char_y)
+                    shape = String(new_x, -(new_y - baseLineShift), char)
+                    self.applyStyleOnShape(shape, node)
+                    if node_name(c) == 'tspan':
+                        self.applyStyleOnShape(shape, c)
+                    gr.add(shape)
+                    last_x = new_x
+                    last_y = new_y
+                    last_char = char
+            else:
+                new_x = (x1 + dx) if has_x else (x + dx0 + sum(frag_lengths[:-1]))
+                new_y = (y1 + dy) if has_y else (y + dy0)
+                shape = String(new_x, -(new_y - baseLineShift), text)
+                self.applyStyleOnShape(shape, node)
+                if node_name(c) == 'tspan':
+                    self.applyStyleOnShape(shape, c)
+                gr.add(shape)
 
         gr.scale(1, -1)
 
