@@ -199,9 +199,12 @@ class TestColorAttrConverter(object):
         mapping = (
             ("red", colors.red),
             ("#ff0000", colors.red),
+            ("#ff000055", colors.Color(1, 0, 0, 1/3.0)),
             ("#f00", colors.red),
-            ("rgb(100%,0.5%,0.5%)", colors.Color(1, 0.005, 0.005, 1)),
+            ("#f00f", colors.red),
+            ("rgb(100%,50%,10%)", colors.Color(1, 1.0/255 * 128, 1.0/255 * 26, 1)),
             ("rgb(255, 0, 0)", colors.red),
+            ("rgba(255, 255, 128, .5)", colors.Color(1, 1, 1.0/255 * 128, .5)),
             ("fuchsia", colors.Color(1, 0, 1, 1)),
             ("slategrey", colors.HexColor(0x708090)),
             ("transparent", colors.Color(0, 0, 0, 0)),
@@ -248,6 +251,8 @@ class TestLengthAttrConverter(object):
             ("20pt", 20 * 1.25),
             ("1.5em", 12 * 1.5),
             ("10.5mm", 10.5*(cm*0.1)),
+            ("3, 5 -7", [3, 5, -7]),
+            ("2pt  12pt", [2 * 1.25, 12 * 1.25]),
         )
         ac = svglib.Svg2RlgAttributeConverter()
         failed = _testit(ac.convertLength, mapping)
@@ -321,12 +326,17 @@ class TestAttrConverter(object):
 
     def test_findAttr(self):
         """
-        Whitespace in attribute values shouldn't disturb parsing.
+        Test various attribute peculiarities.
         """
         ac = svglib.Svg2RlgAttributeConverter()
+        # Whitespace in attribute values shouldn't disturb parsing.
         node = etree.XML('<rect fill=" #00A1DE\n"/>')
         assert ac.findAttr(node, 'fill') == "#00A1DE"
 
+        # Attributes starting with '-' are not supported.
+        node = etree.XML('<text style="-inkscape-font-specification:Arial"/>')
+        assert ac.findAttr(node, '-inkscape-font-specification') == ""
+
     def test_findAttr_parents(self):
         ac = svglib.Svg2RlgAttributeConverter()
         rect_node = etree.XML(
@@ -401,17 +411,20 @@ class TestStyleSheets(object):
               <defs>
                 <style type="text/css">
                 #p1 { fill:rgb(255,0,0); }
+                #p2 { fill:rgb(255,0,0); }
                 .paths { stroke-width:1.5; }
                 </style>
               </defs>
               <g id="g1">
                 <path id="p1" class="paths" d="M 0,-100 V 0 H 50"/>
-                <path id="p2" class="paths other" d="M 0,100 V 0 H 50"/>
+                <path id="p2" class="paths other" style="fill: #000000" d="M 0,100 V 0 H 50"/>
               </g>
             </svg>
         ''')))
         main_group = drawing.contents[0]
         assert main_group.contents[0].contents[0].contents[0].fillColor == colors.red
+        # The style on the element has precedence over the global style
+        assert main_group.contents[0].contents[1].contents[0].fillColor == colors.black
         assert main_group.contents[0].contents[1].contents[0].strokeWidth == 1.5
 
 
